#
# Calling sytax:
#
#   python image.py <param file> [--restart=<restart value>]
#
# optional keyword "restart" can have the following values:
# - readin
# - clean
# - smooth
# - mask
# - reclean
#
# ie: python image.py <param file> --restart=clean
#
# The program will output various miriad directories from each task
# as well as a log (galaxy.image.log) that saves the miriad output.
#
# Steps run by the program:
#
# 1. readin
#    -> *.map [keep]
#    -> *.beam [keep]
#
# 2. clean
#    -> *.clean
#    -> *.restor
#    -> *.cc.restor
#    -> *.resid.restor
#
# 3. smooth
#    -> *.restor.convol
#    -> *.restor.convol.hanning
#    -> *.blank
#
# 4. mask
#    -> *.region [keep?]
#    -> *.mask [keep]
#
# 5. reclean
#    -> *.mclean [keep]
#    -> *.mrestor [keep]
#    -> *.cc.mrestor [keep]
#    -> #.resid.mrestor [keep]
#
# 
# 6. moments
#    -> *.m0
#    -> *.m1
#    -> *.m2
#    -> *.m-2
#
# 7. jvm
#    -> *.spect.txt
#    -> *.spect.ps
#  
#
# 8. cleanup
#    deletes unnecessary files and makes a tarball 
#
# To Add:
#   
#   1. Figure out how to write to a log and have the input scroll
#      (quickly) in the terminal as well, instead of in big chunks.
#  

import os
import sys
import math
import immodules


# check if miriad is able to run:
if os.path.expandvars("$MIRBIN") == "$MIRBIN":
   print ""
   print "MIRIAD tasks cannot be called. " + \
         "Source MIRRC or MIRRC.sh in the MIRIAD directory."
   print ""
   sys.exit(0)


# intialize variables for this scope
pfile = ""
restart = ""
noise = 0.0
lfchans = []
galaxy = ""
log = ""
dniters = 0
mniters = 0


# check calling sequence and set variables
if len(sys.argv) == 2:
    pfile = sys.argv[1]

elif len(sys.argv) == 3:
    pfile = sys.argv[1]

    tmp = sys.argv[2]
    tmp = tmp.split('=')
    value = tmp[1]
    value = value.lower()

    if value == "readin" or value == "clean" or value == "smooth" or \
       value == "mask" or value == "reclean" or value == "moments" \
       or value == "jvm" or value == "cleanup" or value == "recover" \
       or value == "cd":
        restart = value

    else:
        print value + " is not a valid restart point."
        print "Please choose: readin clean smooth mask reclean moments " + \
              "jvm cleanup recover cd"
        sys.exit(0)

else:
    print "Calling sequence: python " + sys.argv[0] + \
          " <param file> [--restart=<step>]"
    sys.exit(0)


# check that parameter file actually exists.
if not os.path.exists(pfile):
    print "Parameter file " + pfile + " does not exist."
    sys.exit(0)


f = open(pfile,mode="r")
# read file and set parameters
for line in f:
   if line[0] == "#":
      # ignore this line
      continue
   else:
      this = line.split()
      
      # search for = in list
      try:
         i = this.index("=")
      except ValueError:
         # this line does not have an =
         continue
      
      if len(this) != 3:
         continue
      
      param = this[0].lower()
      value = this[2].lower()
      
      
      # stupid switch statement
      if param == "galaxy":
         galaxy = value
         
      elif param == "noise":
         if value:
            noise = float(value) # in mJy!

      elif param == "dniters":
         if value:
            dniters = int(value)

      elif param == "mniters":
         if value:
            mniters = int(value)
            
            
      elif param == "lfchans":
         tmp = value.split(',')
         if len(tmp) >= 2:
            lfchans = [int(tmp[0]), int(tmp[1])]
         if len(tmp) >= 4:
            lfchans.append(int(tmp[2]))
            lfchans.append(int(tmp[3]))
         if len(tmp) >= 6:
            lfchans.append(int(tmp[4]))
            lfchans.append(int(tmp[5]))
         if len(tmp) >= 8:
            lfchans.append(int(tmp[6]))
            lfchans.append(int(tmp[7]))
#         if len(tmp) == 2:
#             lfchans = [int(tmp[0]), int(tmp[1])]
#         if len(tmp) == 4:
#             lfchans = [int(tmp[0]), int(tmp[1]), int(tmp[2]), int(tmp[3])]
#         if len(tmp) ==
                        
      else:
          print "Paramter " + param + " is not valid in " + pfile + "."
          sys.exit(0)
                  



if not(galaxy):

    print "No galaxy parameter given."
    print "Add this to the parameter file to continue:"
    print "galaxy = <name>"
    sys.exit(0)


# set up dictionary to keep track of files that will be created.
createdFiles = {"readin":  [galaxy + ".map", \
                            galaxy + ".beam"],
                "clean":   [galaxy + ".clean",\
                            galaxy + ".restor", \
                            galaxy + ".resid.restor", \
                            galaxy + ".restor.linmos", \
                            galaxy + ".cc.restor"], \
                "smooth":  [galaxy + ".cc.restor.convol.hanning"], \
                "mask":    [galaxy + ".region",\
                            galaxy + ".mask"], \
                "reclean": [galaxy + ".mclean", \
                            galaxy + ".mrestor", \
                            galaxy + ".resid.mrestor",\
                            galaxy + ".mrestor.linmos", \
                            galaxy + ".cc.mrestor"],
                "moments": [galaxy + ".m0", \
                            galaxy + ".m1", \
                            galaxy + ".m2", \
                            galaxy + ".m-2"],\
                "jvm":     [galaxy + ".spect.ps",\
                            galaxy + ".spect.txt",\
                            galaxy + ".mrestor.jvm"],
                "keep":    [galaxy + ".mask",\
                            galaxy + ".mrestor",\
                            galaxy + ".mrestor.linmos",\
                            galaxy + ".resid.mrestor",\
                            galaxy + ".cc.mrestor",\
                            galaxy + ".image.log",\
                            galaxy + ".m0",\
                            galaxy + ".m1",\
                            galaxy + ".m2",\
                            galaxy + ".m-2",\
                            galaxy + ".spect.ps",\
                            galaxy + ".spect.txt",\
                            galaxy + ".mrestor.jvm",\
                            galaxy + ".m0.cd"]
               }


# run cleanup first if that is the restart value.
if restart == "cleanup":
    if not(immodules.doStep(createdFiles["keep"])):
        immodules.cleanup(galaxy, createdFiles, pfile)
        print "Done!"
        sys.exit(0)
    else:
        print "Not all files exist to clean up. Run all script steps first."
        sys.exit(0)

if restart == "recover":
   immodules.recover(galaxy, createdFiles["keep"])
   sys.exit(0)



# run column density part - AMS added dec. 15 2009
if restart == "cd":
    if os.path.exists(galaxy + ".beam") and os.path.exists(galaxy + ".m0"):
        if os.path.exists(galaxy + ".m0.cd"):
            os.system("rm -r " + galaxy + ".m0.cd")
        immodules.coldens(galaxy)
    else:
        print "Either beam or m0 file does not exist. Cannot run restart=cd."
        sys.exit(0)


# remove old files.
save = restart
if restart == "readin":
    immodules.remove(createdFiles[restart])
    restart = "clean"

if restart == "clean":
    immodules.remove(createdFiles[restart])
    restart = "smooth"    

if restart == "smooth":
    immodules.remove(createdFiles[restart])
    restart = "mask"    

if restart == "mask":
    immodules.remove(createdFiles[restart])
    restart = "reclean"    

if restart == "reclean":
    immodules.remove(createdFiles[restart])
    restart = "jvm"
    

if restart == "jvm":
    immodules.remove(createdFiles[restart])
    restart = "moments"

if restart == "moments":
    immodules.remove(createdFiles[restart])
         
restart = save


# set up the log.
if restart:
   immodules.resetLog(galaxy, restart)



# now figure out where to start.
if os.path.exists(galaxy + ".map") and os.path.exists(galaxy + ".beam") and \
   os.path.exists(galaxy + ".mask"):
    # no masking needed - it's already done!

    if not(lfchans):
        lfchans = immodules.getLFChans(galaxy)
        
        # append to parameter file for faster reading next time.
        chanstr = ""
        for chan in lfchans:
           chanstr = chanstr + str(chan) + ","
        chanstr = chanstr[0:len(chanstr)-1]
        f = open(pfile, mode="a")
        f.write("lfchans = " + chanstr + "\n")
        f.close()
        

    print "Line Free Channels are: " + str(lfchans)

    if not(noise):
        noise = immodules.getNoise(galaxy, lfchans=lfchans)
        
        # append to parameter file for faster reading next time.
        f = open(pfile, mode="a")
        f.write("noise = " + str(noise) + "\n")
        f.close()

    print "Noise is: " + str(noise) + " mJy."
    
    if immodules.doStep(createdFiles["reclean"]):
        immodules.clean(galaxy, 2.5*noise, mask = galaxy + ".mask", \
                        niters = mniters)
        

        

# otherwise, start from the beginning and see if the files exist.
elif not(os.path.exists(galaxy + ".map")\
   and os.path.exists(galaxy + ".resid.mrestor") \
   and os.path.exists(galaxy + ".cc.mrestor") \
   and os.path.exists(galaxy + ".mrestor") \
   and os.path.exists(galaxy + ".mask")):

    if immodules.doStep(createdFiles["readin"]):
       
        #remove any existing log file
        if os.path.exists(galaxy + ".image.log"):
            os.system("rm -r " + galaxy + ".image.log")

        # check for fits files
        if os.path.exists(galaxy + ".isubim.1.fits") and \
           os.path.exists(galaxy + ".bsubim.1.fits"):
           immodules.readIn(galaxy)

        else:
           if not(os.path.exists(galaxy + ".isubim.1.fits")):
              print "File "+ galaxy + \
                    ".isubim.1.fits does not exist."
           else:
              print "File "+ galaxy + \
                    ".bsubim.1.fits does not exist."
           sys.exit(0)

    # once the map is read in, set lfchans and noise if necessary. Add
    # them to the log.
    if not(lfchans):
        lfchans = immodules.getLFChans(galaxy)

        # append to parameter file for faster reading next time.
        chanstr = ""
        for chan in lfchans:
           chanstr = chanstr + str(chan) + ","
        chanstr = chanstr[0:len(chanstr)-1]
        f = open(pfile, mode="a")
        f.write("lfchans = " + chanstr + "\n")
        f.close()


    print "Line Free Channels are: " + str(lfchans)

    # check if noise is set. find it and append if not.
    if not(noise):
        noise = immodules.getNoise(galaxy, lfchans=lfchans)

        # append to parameter file for faster reading next time.
        f = open(pfile, mode="a")
        f.write("noise = " + str(noise) + "\n")
        f.close()

    print "Noise is: " + str(noise) + " mJy."


    
    if immodules.doStep(createdFiles["clean"]):
        immodules.clean(galaxy, 3*noise, niters = dniters)
    
    if immodules.doStep(createdFiles["smooth"]):
        immodules.smooth(galaxy)
    
    if immodules.doStep(createdFiles["mask"]):
        immodules.mask(galaxy, noise)
    
    if immodules.doStep(createdFiles["reclean"]):
        immodules.clean(galaxy, 2.5*noise, mask = galaxy + ".mask", \
                        niters = mniters)


if os.path.exists(galaxy + ".map")\
   and os.path.exists(galaxy + ".resid.mrestor") \
   and os.path.exists(galaxy + ".cc.mrestor") \
   and os.path.exists(galaxy + ".mrestor") \
   and os.path.exists(galaxy + ".mask"):

   if immodules.doStep(createdFiles["jvm"]):
      immodules.jvm(galaxy)

   if immodules.doStep(createdFiles["moments"]):
      immodules.moments(galaxy, noise)



print "Done!"




   
